import pytest
from bs4 import BeautifulSoup
from seo_analyzer.analyzer import SEOAnalyzer
from unittest.mock import Mock, patch

@pytest.fixture
def analyzer():
    return SEOAnalyzer()

@pytest.fixture
def sample_html():
    return """
    <html>
        <head>
            <title>Test Page</title>
            <meta name="description" content="Test description for SEO analysis">
            <meta name="keywords" content="test,seo,analysis">
            <script type="application/ld+json">
            {
                "@context": "https://schema.org",
                "@type": "Article",
                "name": "Test Article",
                "keywords": ["test", "article", "seo"]
            }
            </script>
        </head>
        <body>
            <h1>Test Heading</h1>
            <p>This is a test paragraph about SEO analysis.</p>
            <img src="test.jpg" alt="Test image">
            <img src="no-alt.jpg">
            <a href="https://example.com">External Link</a>
            <a href="/internal">Internal Link</a>
        </body>
    </html>
    """

@pytest.fixture
def soup(sample_html):
    return BeautifulSoup(sample_html, 'html.parser')

def test_extract_keywords_from_sources(analyzer, soup):
    keywords = analyzer._extract_keywords_from_sources(soup)
    assert isinstance(keywords, list)
    assert 'test' in keywords
    assert 'seo' in keywords
    assert 'analysis' in keywords
    assert 'article' in keywords

def test_analyze_meta_data(analyzer, soup):
    meta_data = analyzer._analyze_meta_data(soup)
    assert meta_data['title'] == 'Test Page'
    assert meta_data['description'] == 'Test description for SEO analysis'
    assert meta_data['keywords'] == 'test,seo,analysis'

def test_analyze_headers(analyzer, soup):
    headers = analyzer._analyze_headers(soup)
    assert headers['h1'] == 1
    assert headers['h2'] == 0

def test_analyze_images(analyzer, soup):
    images = analyzer._analyze_images(soup)
    assert images['total'] == 2
    assert images['missing_alt'] == 1
    assert len(images['missing_alt_details']) == 1
    assert images['missing_alt_details'][0]['src'] == 'no-alt.jpg'

def test_analyze_links(analyzer, soup):
    links = analyzer._analyze_links(soup, 'https://example.com')
    assert len(links['internal_links']) == 1
    assert len(links['external_links']) == 1
    assert 'https://example.com/internal' in links['internal_links']
    assert 'https://example.com' in links['external_links']

def test_analyze_keyword_density(analyzer, soup):
    density = analyzer._analyze_keyword_density(soup)
    assert isinstance(density, dict)
    assert 'total_words' in density
    assert 'keyword_stats' in density
    assert 'general_density' in density
    assert 'combined_density_score' in density
    assert 'combined_density_percentage' in density
    assert density['total_words'] > 0
    
    # Test keyword stats
    assert 'test' in density['keyword_stats']
    test_stats = density['keyword_stats']['test']
    assert 'count' in test_stats
    assert 'density_percentage' in test_stats
    assert 'density' in test_stats
    assert 'density_score' in test_stats
    assert 'locations' in test_stats
    
    # Test general density
    assert len(density['general_density']) <= 10  # Top 10 most common words
    for word_stats in density['general_density'].values():
        assert 'count' in word_stats
        assert 'density_percentage' in word_stats
        assert 'density' in word_stats
        assert 'density_score' in word_stats
    
    # Test combined metrics
    assert isinstance(density['combined_density_score'], float)
    assert isinstance(density['combined_density_percentage'], float)
    assert density['combined_density_score'] == density['combined_density_percentage'] / 2.5

def test_get_canonical_url(analyzer, soup):
    canonical = analyzer._get_canonical_url(soup)
    assert canonical is None

@patch('requests.get')
def test_get_page_content(mock_get, analyzer):
    mock_response = Mock()
    mock_response.text = '<html><body>Test</body></html>'
    mock_response.raise_for_status = Mock()
    mock_get.return_value = mock_response

    response, soup = analyzer._get_page_content('https://example.com')
    assert isinstance(soup, BeautifulSoup)
    assert mock_get.called
    # Verify that a request was made with a User-Agent header
    args, kwargs = mock_get.call_args
    assert args[0] == 'https://example.com'
    assert 'headers' in kwargs
    assert 'User-Agent' in kwargs['headers']

def test_analyze_structured_data(analyzer, soup):
    data = analyzer._analyze_structured_data(soup)
    assert isinstance(data, dict)
    assert len(data['json_ld']) == 1
    assert data['json_ld'][0]['@type'] == 'Article'
    assert len(data['recommendations']) > 0

def test_generate_recommendations(analyzer, soup):
    metrics = {
        'meta_data': {
            'title': 'Short',
            'description': None
        },
        'headers': {'h1': 2},
        'images': {
            'total': 2,
            'missing_alt': 1,
            'missing_alt_details': [{'src': 'test.jpg'}]
        },
        'structured_data': {'recommendations': []},
        'mobile_friendliness': {'recommendations': []}
    }
    
    recommendations = analyzer._generate_recommendations(metrics)
    assert isinstance(recommendations, dict)
    assert len(recommendations['meta']) > 0
    assert len(recommendations['headers']) > 0
    assert len(recommendations['images']) > 0